# Generated by Django 2.2.28 on 2023-05-02 07:40

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import djangoldp.fields
from djangoldp_account.models import LDPUser

from ..models import Avatar
from ..permissions.avatar import AvatarPermissions


def create_avatars(apps, schema_editor):
    users = LDPUser.objects.all()
    i = 0
    for user in users:
        avatar = Avatar(creator=user, name_avatar='chenille', name_number=i)
        avatar.save()
        i += 1

def update_avatar_number(apps, schema_editor):
    if not schema_editor.connection.vendor.startswith('postgres'):
        return
    schema_editor.execute("UPDATE djangoldp_needle_avatar SET name_number = floor(random() * (9999999 - 1111111 + 1) + 1111111)::int")

class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp_needle', '0029_auto_20230411_1211'),
    ]

    operations = [
        migrations.CreateModel(
            name='Avatar',
            fields=[
                ('urlid', djangoldp.fields.LDPUrlField(blank=True, null=True, unique=True)),
                ('is_backlink', models.BooleanField(default=False, help_text='set automatically to indicate the Model is a backlink')),
                ('allow_create_backlink', models.BooleanField(default=True, help_text='set to False to disable backlink creation after Model save')),
                ('creator', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='avatar', serialize=False, to=settings.AUTH_USER_MODEL)),
                ('name_number', models.CharField(max_length=255)),
                ('name_avatar', models.CharField(max_length=255)),
                ('name_quality', models.CharField(max_length=255, null=True)),
                ('name', models.CharField(max_length=16)),
            ],
            options={
                'abstract': False,
                'default_permissions': ['add', 'change', 'delete', 'view', 'control'],
                'rdf_type': 'hd:avatar',
                'owner_field': 'creator',
                'permission_classes': [AvatarPermissions],
                'depth': 0,
                'anonymous_perms': [],
                'authenticated_perms': ['view'],
                'owner_perms': ['view', 'change'],
            },
        ),
        migrations.RunPython(create_avatars),
        migrations.RunPython(update_avatar_number),
    ]
